@@ -8,6 +8,7 @@ from django.db import models
 from django.utils.translation import ugettext_lazy as _
 from django_extensions.db.models import TimeStampedModel
 from django.core.exceptions import ObjectDoesNotExist
+from model_utils import FieldTracker
 
 from . import api
 
@@ -54,6 +55,7 @@ class SyncJob(models.Model):
     added = models.PositiveIntegerField(null=True)
     updated = models.PositiveIntegerField(null=True)
     deleted = models.PositiveIntegerField(null=True)
+    skipped = models.PositiveIntegerField(null=True)
     success = models.NullBooleanField()
     message = models.TextField(blank=True, null=True)
     sync_type = models.CharField(max_length=32, default='full')
@@ -94,6 +96,7 @@ class ConnectWiseBoard(TimeStampedModel):
 
     objects = models.Manager()
     available_objects = AvailableConnectWiseBoardManager()
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name',)
@@ -183,6 +186,8 @@ class BoardStatus(TimeStampedModel):
     objects = models.Manager()
     available_objects = AvailableBoardStatusManager()
 
+    tracker = FieldTracker()
+
     class Meta:
         ordering = ('board__name', 'sort_order', 'name')
         verbose_name_plural = 'Board statuses'
@@ -213,6 +218,8 @@ class Location(TimeStampedModel):
     name = models.CharField(max_length=30)
     where = models.CharField(max_length=100, blank=True, null=True)
 
+    tracker = FieldTracker()
+
     class Meta:
         ordering = ('name',)
 
@@ -254,6 +261,8 @@ class Member(TimeStampedModel):
     objects = models.Manager()
     regular_objects = RegularMemberManager()
 
+    tracker = FieldTracker()
+
     class Meta:
         ordering = ('first_name', 'last_name')
 
@@ -318,6 +327,7 @@ class Company(TimeStampedModel):
 
     objects = models.Manager()
     available_objects = AvailableCompanyManager()
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'companies'
@@ -344,6 +354,7 @@ class CompanyStatus(models.Model):
     custom_note_flag = models.BooleanField()
     cancel_open_tracks_flag = models.BooleanField()
     track_id = models.PositiveSmallIntegerField(blank=True, null=True)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'Company statuses'
@@ -355,6 +366,7 @@ class CompanyStatus(models.Model):
 class CompanyType(models.Model):
     name = models.CharField(max_length=50)
     vendor_flag = models.BooleanField()
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -370,6 +382,7 @@ class MyCompanyOther(models.Model):
         null=True,
         on_delete=models.SET_NULL
         )
+    tracker = FieldTracker()
 
 
 class Calendar(models.Model):
@@ -409,6 +422,8 @@ class Calendar(models.Model):
     sunday_end_time = models.TimeField(auto_now=False, auto_now_add=False,
                                        blank=True, null=True)
 
+    tracker = FieldTracker()
+
     def __str__(self):
         return self.name
 
@@ -674,6 +689,8 @@ class Holiday(models.Model):
     holiday_list = models.ForeignKey(
         'HolidayList', on_delete=models.CASCADE)
 
+    tracker = FieldTracker()
+
     def __str__(self):
         return self.name
 
@@ -681,6 +698,8 @@ class Holiday(models.Model):
 class HolidayList(models.Model):
     name = models.CharField(max_length=200)
 
+    tracker = FieldTracker()
+
     def __str__(self):
         return self.name
 
@@ -689,6 +708,8 @@ class ScheduleType(models.Model):
     name = models.CharField(max_length=50)
     identifier = models.CharField(max_length=1)
 
+    tracker = FieldTracker()
+
     class Meta:
         ordering = ('name', )
 
@@ -698,6 +719,7 @@ class ScheduleType(models.Model):
 
 class ScheduleStatus(models.Model):
     name = models.CharField(max_length=30)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'Schedule statuses'
@@ -743,6 +765,7 @@ class ScheduleEntry(models.Model):
         null=True,
         on_delete=models.SET_NULL
     )
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'Schedule entries'
@@ -761,6 +784,7 @@ class ScheduleEntry(models.Model):
 
 class Territory(models.Model):
     name = models.CharField(max_length=250, blank=True, null=True)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'Territories'
@@ -815,6 +839,7 @@ class TimeEntry(models.Model):
         'WorkType', blank=True, null=True, on_delete=models.SET_NULL)
     agreement = models.ForeignKey(
         'Agreement', blank=True, null=True, on_delete=models.SET_NULL)
+    tracker = FieldTracker()
 
     def get_entered_time(self):
         if self.time_end:
@@ -843,6 +868,7 @@ class Team(TimeStampedModel):
 
     objects = models.Manager()
     available_objects = AvailableBoardTeamManager()
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'Teams'
@@ -861,6 +887,7 @@ class TicketPriority(TimeStampedModel):
     _color = models.CharField(
         max_length=50, null=True, blank=True, db_column='color'
     )
+    tracker = FieldTracker()
 
     DEFAULT_COLORS = {
         '1': 'red',
@@ -910,6 +937,8 @@ class ProjectStatus(TimeStampedModel):
     inactive_flag = models.BooleanField(default=False)
     closed_flag = models.BooleanField(default=False)
 
+    tracker = FieldTracker()
+
     class Meta:
         ordering = ('name', )
         verbose_name_plural = 'Project statuses'
@@ -923,6 +952,8 @@ class ProjectType(TimeStampedModel):
     default_flag = models.BooleanField(default=False)
     inactive_flag = models.BooleanField(default=False)
 
+    tracker = FieldTracker()
+
     class Meta:
         ordering = ('name', )
         verbose_name_plural = 'Project types'
@@ -954,6 +985,7 @@ class ProjectPhase(TimeStampedModel):
     board = models.ForeignKey(
         'ConnectwiseBoard', blank=True, null=True, on_delete=models.SET_NULL
     )
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'Project phases'
@@ -975,6 +1007,7 @@ class ProjectTeamMember(TimeStampedModel):
     work_role = models.ForeignKey(
         'WorkRole', null=True, on_delete=models.SET_NULL
     )
+    tracker = FieldTracker()
 
     def __str__(self):
         return '{}/{}'.format(self.id, self.member)
@@ -1037,6 +1070,7 @@ class Project(TimeStampedModel):
 
     objects = models.Manager()
     available_objects = AvailableProjectManager()
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -1067,6 +1101,7 @@ class Project(TimeStampedModel):
 
 class OpportunityStage(TimeStampedModel):
     name = models.CharField(max_length=50)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -1094,6 +1129,7 @@ class OpportunityStatus(TimeStampedModel):
 
     objects = models.Manager()
     available_objects = AvailableOpportunityStatusManager()
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -1105,6 +1141,7 @@ class OpportunityStatus(TimeStampedModel):
 
 class OpportunityPriority(TimeStampedModel):
     name = models.CharField(max_length=50)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -1117,6 +1154,7 @@ class OpportunityPriority(TimeStampedModel):
 class OpportunityType(TimeStampedModel):
     description = models.CharField(max_length=50)
     inactive_flag = models.BooleanField(default=False)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('description', )
@@ -1166,6 +1204,7 @@ class Opportunity(TimeStampedModel):
     opportunity_type = models.ForeignKey('OpportunityType',
                                          blank=True, null=True,
                                          on_delete=models.SET_NULL)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -1339,6 +1378,7 @@ class Ticket(TimeStampedModel):
     agreement = models.ForeignKey(
         'Agreement', blank=True, null=True, related_name='agreement_tickets',
         on_delete=models.SET_NULL)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name = 'Ticket'
@@ -1519,6 +1559,7 @@ class ServiceNote(TimeStampedModel):
     ticket = models.ForeignKey('Ticket', on_delete=models.CASCADE)
     member = models.ForeignKey(
         'Member', blank=True, null=True, on_delete=models.SET_NULL)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('-date_created', 'id')
@@ -1551,6 +1592,7 @@ class Sla(TimeStampedModel, SlaGoalsMixin):
         null=True,
         on_delete=models.SET_NULL
         )
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'SLAs'
@@ -1587,6 +1629,7 @@ class SlaPriority(TimeStampedModel, SlaGoalsMixin):
     respond_hours = models.FloatField()
     plan_within = models.FloatField()
     resolution_hours = models.FloatField()
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'SLA Priorities'
@@ -1601,6 +1644,7 @@ class OpportunityNote(TimeStampedModel):
     text = models.TextField(blank=True, null=True, max_length=2000)
     date_created = models.DateTimeField(blank=True, null=True)
     opportunity = models.ForeignKey('Opportunity', on_delete=models.CASCADE)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('-date_created', 'id', )
@@ -1617,6 +1661,7 @@ class ActivityStatus(TimeStampedModel):
     inactive_flag = models.BooleanField(default=False)
     spawn_followup_flag = models.BooleanField(default=False)
     closed_flag = models.BooleanField(default=False)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -1634,6 +1679,7 @@ class ActivityType(TimeStampedModel):
     email_flag = models.BooleanField(default=False)
     memo_flag = models.BooleanField(default=False)
     history_flag = models.BooleanField(default=False)
+    tracker = FieldTracker()
 
     class Meta:
         ordering = ('name', )
@@ -1661,6 +1707,7 @@ class Activity(TimeStampedModel):
         'Company', blank=True, null=True, on_delete=models.CASCADE)
     agreement = models.ForeignKey(
         'Agreement', blank=True, null=True, on_delete=models.SET_NULL)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'activities'
@@ -1703,6 +1750,7 @@ class Activity(TimeStampedModel):
 
 class SalesProbability(TimeStampedModel):
     probability = models.IntegerField()
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name_plural = 'Sales probabilities'
@@ -1851,6 +1899,7 @@ class StateMachineManager(object):
 class Type(TimeStampedModel):
     name = models.CharField(max_length=50)
     board = models.ForeignKey('ConnectWiseBoard', on_delete=models.CASCADE)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name = 'Type'
@@ -1863,6 +1912,7 @@ class Type(TimeStampedModel):
 class SubType(TimeStampedModel):
     name = models.CharField(max_length=50)
     board = models.ForeignKey('ConnectWiseBoard', on_delete=models.CASCADE)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name = 'Subtype'
@@ -1875,6 +1925,7 @@ class SubType(TimeStampedModel):
 class Item(TimeStampedModel):
     name = models.CharField(max_length=50)
     board = models.ForeignKey('ConnectWiseBoard', on_delete=models.CASCADE)
+    tracker = FieldTracker()
 
     class Meta:
         verbose_name = 'Item'
@@ -1891,6 +1942,7 @@ class WorkType(TimeStampedModel):
     bill_time = models.CharField(
         max_length=50, choices=BILL_TYPES, blank=True, null=True
     )
+    tracker = FieldTracker()
 
     def __str__(self):
         return self.name
@@ -1899,6 +1951,7 @@ class WorkType(TimeStampedModel):
 class WorkRole(TimeStampedModel):
     name = models.CharField(max_length=50)
     inactive_flag = models.BooleanField(default=False)
+    tracker = FieldTracker()
 
     def __str__(self):
         return self.name
@@ -1918,6 +1971,7 @@ class Agreement(TimeStampedModel):
         'WorkRole', null=True, on_delete=models.SET_NULL)
     company = models.ForeignKey(
         'Company', null=True, on_delete=models.SET_NULL)
+    tracker = FieldTracker()
 
     def __str__(self):
         return '{}/{}'.format(self.agreement_type, self.name)
